/*
* Copyright © 2024 ESWIN. All rights reserved.
* Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
* all intellectual property and proprietary rights in and to this software. Except as expressly
* authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
* modified, adapted, translated, or created derivative work of, in whole or in part.
*
* File Name : es_cipher.h
*
*/
/**
* \file
* \brief Describes the API about the cipher.
*/

#ifndef __ES_CIPHER_H__
#define __ES_CIPHER_H__

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/time.h>
#include <poll.h>

#define __USE_GNU
#include <sched.h>
#include <pthread.h>

#include <essdk/es_type.h>
#include <essdk/es_common.h>
#include <essdk/es_errno.h>

#ifdef ES_CIPHER_QEMU_DEBUG
typedef unsigned int		dma_addr_t;
#else
typedef unsigned long		dma_addr_t;
#endif

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif /* __cplusplus */

#define ES_CIPHER_ERROR_BASE	ES_DEF_ERR(ES_ID_CIPHER, EN_ERR_LEVEL_ERROR, 0)
#define ES_CIPHER_ERROR_NUM(X)	(ES_CIPHER_ERROR_BASE + X)

#define  ES_ERR_CIPHER_INVALID_PARA			ES_CIPHER_ERROR_NUM(EN_ERR_ILLEGAL_PARAM)
#define  ES_ERR_CIPHER_NO_MEM				ES_CIPHER_ERROR_NUM(EN_ERR_NOMEM)
#define  ES_ERR_CIPHER_NOT_INIT				ES_CIPHER_ERROR_NUM(64)
#define  ES_ERR_CIPHER_FAILED_INIT			ES_CIPHER_ERROR_NUM(65)
#define  ES_ERR_CIPHER_OVERFLOW				ES_CIPHER_ERROR_NUM(66)
#define  ES_ERR_CIPHER_INVALID_HANDLE			ES_CIPHER_ERROR_NUM(67)
#define  ES_ERR_CIPHER_FAILED_GETHANDLE			ES_CIPHER_ERROR_NUM(68)
#define  ES_ERR_CIPHER_FAILED_RELEASEHANDLE		ES_CIPHER_ERROR_NUM(69)
#define  ES_ERR_CIPHER_HW_ERROR				ES_CIPHER_ERROR_NUM(70)
#define  ES_ERR_CIPHER_SERVICE_FAILED			ES_CIPHER_ERROR_NUM(71)
#define  ES_ERR_CIPHER_IOVA				ES_CIPHER_ERROR_NUM(72)

/* OTP Field Macro Definition*/
#define OTP_RESERVED_FOR_OTHER_KEYS_ADDR         (0x738)
#define OTP_RESERVED_FOR_OTHER_KEYS_SIZE         (128)
#define OTP_RESERVED_FOR_USER_SECURITY_DATA_ADDR (0x7b8)
#define OTP_RESERVED_FOR_USER_SECURITY_DATA_SIZE (72)
#define OTP_SECURITY_ENABLE_CONTROL_ADDR         (0x800)
#define OTP_SECURITY_ENABLE_CONTROL_SIZE         (1)
#define OTP_WHOLE_OTP_PROGRAMMING_DISABLE_ADDR   (0x801)
#define OTP_WHOLE_OTP_PROGRAMMING_DISABLE_SIZE   (1)
#define OTP_OTP_PAD_ACCESS_DISABLE_ADDR          (0x802)
#define OTP_OTP_PAD_ACCESS_DISABLE_SIZE          (1)
#define OTP_DEVELOPER_MODE_ADDR                  (0x803)
#define OTP_DEVELOPER_MODE_SIZE                  (1)
#define OTP_RESERVED0_ADDR                       (0x804)
#define OTP_RESERVED0_SIZE                       (8)
#define OTP_MARKET_ID_ADDR                       (0x80c)
#define OTP_MARKET_ID_SIZE                       (32)
#define OTP_DEVICE_ID_ADDR                       (0x82c)
#define OTP_DEVICE_ID_SIZE                       (32)
#define OTP_FUNCTIONAL_FEATURE_LIST_ADDR         (0x84c)
#define OTP_FUNCTIONAL_FEATURE_LIST_SIZE         (16)
#define OTP_SECURITY_FEATURE_LIST_ADDR           (0x85c)
#define OTP_SECURITY_FEATURE_LIST_SIZE           (16)
#define OTP_RESERVED_FOR_OTHER_DATA_ADDR         (0x86c)
#define OTP_RESERVED_FOR_OTHER_DATA_SIZE         (916)
#define OTP_UNIQUE_CHIP_ID_ADDR                  (0xc00)
#define OTP_UNIQUE_CHIP_ID_SIZE                  (16)
#define OTP_NUMBER_OF_DIE_ADDR                   (0xc10)
#define OTP_NUMBER_OF_DIE_SIZE                   (1)
#define OTP_DIE_ORDINAL_ADDR                     (0xc11)
#define OTP_DIE_ORDINAL_SIZE                     (1)
#define OTP_RESERVED1_ADDR                       (0xc12)
#define OTP_RESERVED1_SIZE                       (110)
#define OTP_DFT_INFORMATION_ADDR                 (0xc80)
#define OTP_DFT_INFORMATION_SIZE                 (256)
#define OTP_SERVICE_ADDR_EDGE                 	 (0xd80) // the max addr(access to this addr is invalid)
/*************************** Structure Definition ****************************/
/** \addtogroup      CIPHER */
/** @{ */  /** <!-- [CIPHER] */

/** max length of SM2, unit: word */
#define SM2_LEN_IN_WROD			(8)

/** max length of SM2, unit: byte */
#define SM2_LEN_IN_BYTE			(SM2_LEN_IN_WROD * 4)

/** CIPHER set IV for first package */
#define CIPHER_IV_CHANGE_ONE_PKG	(1)

/** CIPHER set IV for first package */
#define CIPHER_IV_CHANGE_ALL_PKG	(2)

/** TRNG MAX len per ES_CIPHER_GetRandomNumber request */
#define MAX_TRNG_LEN		(0x10000)

/*! \hash result max size in byte*/
#define HASH_RESULT_MAX_SIZE	(64)

/** MAX Eliptic Curve Cryptography key len */
#define MAX_ECC_KEY_LEN		(66)	// When PKA_SW_CURVE_NIST_P512 is used

/** MIN Eliptic Curve Cryptography key len */
#define MIN_ECC_KEY_LEN		(32)

/** Eliptic Curve Cryptography symc key len */
#define ECC_SYMC_KEY_LEN	(64)

#define AES_BLOCK_SIZE		(16)

#define DES_BLOCK_SIZE		(8)

#define SM4_BLOCK_SIZE		(16)

#define SIGN_FIXED_SIZE		256

/** Cipher crypto mode, including algorithm and work mode */
typedef enum esES_CIPHER_ALG_MODE_E
{
  ES_CIPHER_ALG_MODE_AES_ECB = 0x4,
  ES_CIPHER_ALG_MODE_AES_CBC,
  ES_CIPHER_ALG_MODE_AES_CTR,
  ES_CIPHER_ALG_MODE_AES_CCM,
  ES_CIPHER_ALG_MODE_AES_GCM,
  ES_CIPHER_ALG_MODE_AES_F8,	// not support
  ES_CIPHER_ALG_MODE_AES_XTS,	// not support
  ES_CIPHER_ALG_MODE_AES_CFB,
  ES_CIPHER_ALG_MODE_AES_OFB,
  ES_CIPHER_ALG_MODE_AES_CS1,
  ES_CIPHER_ALG_MODE_AES_CS2,
  ES_CIPHER_ALG_MODE_AES_CS3,
  ES_CIPHER_ALG_MODE_MULTI2_ECB,	// not support
  ES_CIPHER_ALG_MODE_MULTI2_CBC,	// not support
  ES_CIPHER_ALG_MODE_MULTI2_OFB,	// not support
  ES_CIPHER_ALG_MODE_MULTI2_CFB,	// not support
  ES_CIPHER_ALG_MODE_3DES_CBC = 0x14,
  ES_CIPHER_ALG_MODE_3DES_ECB,
  ES_CIPHER_ALG_MODE_DES_CBC,
  ES_CIPHER_ALG_MODE_DES_ECB,
  ES_CIPHER_ALG_MODE_SM4_ECB = 0x1e,
  ES_CIPHER_ALG_MODE_SM4_CBC,
  ES_CIPHER_ALG_MODE_SM4_CFB,
  ES_CIPHER_ALG_MODE_SM4_OFB,
  ES_CIPHER_ALG_MODE_SM4_CTR,
  ES_CIPHER_ALG_MODE_SM4_CCM,
  ES_CIPHER_ALG_MODE_SM4_GCM,
  ES_CIPHER_ALG_MODE_SM4_F8,	// not support
  ES_CIPHER_ALG_MODE_SM4_XTS,	// not support
  ES_CIPHER_ALG_MODE_SM4_CS1,
  ES_CIPHER_ALG_MODE_SM4_CS2,
  ES_CIPHER_ALG_MODE_SM4_CS3,

  ES_CIPHER_ALG_MODE_HASH_MD5 = 42,
  ES_CIPHER_ALG_MODE_HMAC_MD5,
  ES_CIPHER_ALG_MODE_HASH_SHA1,
  ES_CIPHER_ALG_MODE_HMAC_SHA1,
  ES_CIPHER_ALG_MODE_HASH_SHA224,
  ES_CIPHER_ALG_MODE_HMAC_SHA224,
  ES_CIPHER_ALG_MODE_HASH_SHA256,
  ES_CIPHER_ALG_MODE_HMAC_SHA256,
  ES_CIPHER_ALG_MODE_HASH_SHA384,
  ES_CIPHER_ALG_MODE_HMAC_SHA384,
  ES_CIPHER_ALG_MODE_HASH_SHA512,
  ES_CIPHER_ALG_MODE_HMAC_SHA512,
  ES_CIPHER_ALG_MODE_HASH_SHA512_224,
  ES_CIPHER_ALG_MODE_HMAC_SHA512_224,
  ES_CIPHER_ALG_MODE_HASH_SHA512_256,
  ES_CIPHER_ALG_MODE_HMAC_SHA512_256,
  ES_CIPHER_ALG_MODE_MAC_XCBC,
  ES_CIPHER_ALG_MODE_MAC_CMAC,
  ES_CIPHER_ALG_MODE_HASH_CRC32 = 66,
  ES_CIPHER_ALG_MODE_HASH_SM3 = 80,
  ES_CIPHER_ALG_MODE_HMAC_SM3,
  ES_CIPHER_ALG_MODE_MAC_SM4_XCBC,
  ES_CIPHER_ALG_MODE_MAC_SM4_CMAC,
} ES_CIPHER_ALG_MODE_E;

/** Key length */
typedef enum esES_CIPHER_ENC_KEY_LENGTH_E
{
	ES_CIPHER_KEY_AES_128BIT	= 16,  /**< 128-bit key for the AES algorithm */
	ES_CIPHER_KEY_AES_192BIT	= 24,  /**< 192-bit key for the AES algorithm */
	ES_CIPHER_KEY_AES_256BIT	= 32,  /**< 256-bit key for the AES algorithm */
	ES_CIPHER_KEY_DES_3KEY		= 24,  /**< Three keys for the DES algorithm, TDES_KEY_192BIT */
	ES_CIPHER_KEY_SM_128BIT		= 16,  /**< SM1, SM4 */
	ES_CIPHER_KEY_DEFAULT		= 8,  /**< default key length, DES-8, SM1-48, SM4-16 */
	ES_CIPHER_KEY_LEN_MAX		= 32,
	ES_CIPHER_KEY_INVALID		= 0xffffffff,
}ES_CIPHER_ENC_KEY_LENGTH_E;

/** SYMC Enc&Dec Key soure selecting parameters */
typedef enum esES_CIPHER_SYMC_KEY_SRC_E
{
	ES_CIPHER_SYMC_KEY_SRC_INTERNAL	= 0x0,  /**< Internal ROM Key*/
	ES_CIPHER_SYMC_KEY_SRC_USER	= 0xFF, /**< External User Key*/
	ES_CIPHER_SYMC_KEY_SRC_INVALID	= 0xffffffff,
}ES_CIPHER_SYMC_KEY_SRC_E;


/** Structure of the cipher control information for AES/SM4/DES/3DES encrypt or decrypt */
#define MAX_IV_LEN 32
typedef struct esES_CIPHER_CTRL_S
{
	ES_U32 u32Key[8];                               /**< Key input */
	ES_U32 u32IV[8];                                /**< Initialization vector (IV) , MAX IV_LEN is 32 Bytes */
	ES_CIPHER_SYMC_KEY_SRC_E enKeySrc;               /**< Select key source: internal, or from u32Kye[8] as external user key */
	ES_CIPHER_ALG_MODE_E enAlgMode;                 /**< Cipher algorithm */
	ES_CIPHER_ENC_KEY_LENGTH_E enKeyLen;            /**< Key length */
	ES_U32 u32IvLen;
} ES_CIPHER_CTRL_S;


/** Hash init struct input */
typedef struct
{
	ES_U8 *pu8HMACKey;
	ES_U32 u32HMACKeyLen;
	ES_CIPHER_ALG_MODE_E enShaType;
}ES_CIPHER_HASH_CTRL_S;

/* rsa key length */
#define RSA_MIN_KEY_LEN		(128)
#define RSA_MAX_KEY_LEN		(512)
typedef enum esES_CIPHER_RSA_PUB_PRI_OP_E
{
	ES_CIPHER_RSA_PUB_OP = 0x0,	/*  RSA public key operation*/
	ES_CIPHER_RSA_PRI_OP,		/*  RSA private key operation*/
}ES_CIPHER_RSA_PUB_PRI_OP_E;

/* RSA SCHEME  */
typedef enum esES_CIPHER_RSA_ENC_SCHEME_E
{
	ES_CIPHER_RSA_SCHEME_RSA1024 = 0,
	ES_CIPHER_RSA_SCHEME_RSA2048,
	ES_CIPHER_RSA_SCHEME_RSA4096,
	ES_CIPHER_RSA_SCHEME_ECDSA,
	ES_CIPHER_RSA_SCHEME_PLAINTEXT,
}ES_CIPHER_RSA_ENC_SCHEME_E;

/** RSA Key soure selecting parameters */
typedef enum esES_CIPHER_RSA_KEY_SRC_E
{
	ES_CIPHER_RSA_PUB_KEY_SRC_OTP	= 0x0,  /**< Internal OTP Key*/

	ES_CIPHER_RSA_PUB_KEY_SRC_ROM_1, /**< Internal ROM Key 1*/
	ES_CIPHER_RSA_PUB_KEY_SRC_ROM_2, /**< Internal ROM Key 2*/
	ES_CIPHER_RSA_PUB_KEY_SRC_ROM_3, /**< Internal ROM Key 3*/
	ES_CIPHER_RSA_PUB_KEY_SRC_ROM_4, /**< Internal ROM Key 4*/
	ES_CIPHER_RSA_PUB_KEY_SRC_ROM_5, /**< Internal ROM Key 5*/
	ES_CIPHER_RSA_PUB_KEY_SRC_ROM_6, /**< Internal ROM Key 6*/
	ES_CIPHER_RSA_PUB_KEY_SRC_ROM_7, /**< Internal ROM Key 7*/

	ES_CIPHER_RSA_PUB_KEY_SRC_RAM	= 8, /**< Internal ram Key*/

	ES_CIPHER_RSA_PRIV_KEY_SRC_EXT	= 254, /**< External private Key*/
	ES_CIPHER_RSA_PUB_KEY_SRC_EXT	= 255, /**< External public Key*/

	ES_CIPHER_RSA_KEY_SRC_INVALID	= 0xffffffff,
}ES_CIPHER_RSA_KEY_SRC_E;

typedef struct
{
	ES_U32 u32NLen;	/**< length of public modulus(i.e, keylen), max value is 512Byte*/
	ES_U32 u32E;	/**< public exponent */
	ES_U8  *pu8N;	/**< point to public modulus  */
}ES_CIPHER_RSA_PUB_KEY_S;

/** RSA public key encryption struct input */
typedef struct
{
	ES_CIPHER_RSA_ENC_SCHEME_E	enScheme;	/** RSA encryption scheme*/
	ES_CIPHER_RSA_KEY_SRC_E		enKeySrc;	/** RSA key source*/
	ES_CIPHER_RSA_PUB_KEY_S		stPubKey;	/** RSA public key struct */
}ES_CIPHER_RSA_PUB_CTRL_S;


/** RSA private key struct */
typedef struct
{
	ES_U32 u32NLen;	/**< length of public modulus(i.e, keylen), max value is 512Byte*/
	ES_U8 *pu8D;	/*!<  private exponent  */
	ES_U8 *pu8N;	/*!<  public modulus    */
}ES_CIPHER_RSA_PRI_KEY_S;

/** RSA private key decryption struct input */
typedef struct
{
	ES_CIPHER_RSA_ENC_SCHEME_E	enScheme;	/** RSA encryption scheme */
	ES_CIPHER_RSA_KEY_SRC_E		enKeySrc;	/** RSA key source*/
	ES_CIPHER_RSA_PRI_KEY_S		stPriKey;	/** RSA private key struct */
}ES_CIPHER_RSA_PRI_CTRL_S;

/** RSA signature verify struct input */
typedef struct
{
	ES_CIPHER_RSA_ENC_SCHEME_E	enScheme;	/** RSA signature scheme*/
	ES_CIPHER_RSA_KEY_SRC_E		enKeySrc;	/** RSA key source*/
 }ES_CIPHER_NSIGN_VERIFY_S;

/** ECC(eliptic curve cryptography)  Diffie–Hellman key Exchange */
typedef enum esES_CIHPER_ECC_CURVE_TYPE_E
{
	ES_CIPHER_PKA_SW_CURVE_NIST_P256 = 0,
	ES_CIPHER_PKA_SW_CURVE_NIST_P384,
	ES_CIPHER_PKA_SW_CURVE_NIST_P521,
	ES_CIPHER_PKA_SW_CURVE_BRAINPOOL_P256R1,
	ES_CIPHER_PKA_SW_CURVE_BRAINPOOL_P384R1,
	ES_CIPHER_PKA_SW_CURVE_BRAINPOOL_P512R1,
	ES_CIPHER_PKA_SW_CURVE_INVALID
}ES_CIHPER_ECC_CURVE_TYPE_E;
static const ES_U8 es_pka_curve_key_len[ES_CIPHER_PKA_SW_CURVE_INVALID] = {32, 48, 66, 32, 48, 64};

typedef struct
{
	ES_U32 keylen;
	ES_U8 *pu8X;
	ES_U8 *pu8Y;
	ES_U8 *pu8Z;
}ES_CIPHER_ECDH_KEY_S;

typedef struct
{
	ES_U32 keylen;
	ES_U8 *pu8X;
	ES_U8 *pu8Y;
}ES_CIPHER_ECDH_PUB_KEY_S;

typedef struct
{
	ES_U32 keylen;
	ES_U8 *pu8Z;
}ES_CIPHER_ECDH_PRI_KEY_S;

/** ECC(eliptic curve cryptography)  Diffie–Hellman key Exchange */
typedef enum esES_CIHPER_LOADABLE_SERVICE_TYPE_E
{
	ES_CIPHER_LOADABLE_REGISTER = 0,
	ES_CIPHER_LOADABLE_UNREGISTER,
	ES_CIPHER_LOADABLE_IOCTL,
}ES_CIHPER_LOADABLE_SERVICE_TYPE_E;

typedef enum esES_CIPHER_OTP_XOR_E
{
	ES_CIPHER_OTP_XOR_DISABLE = 0,
	ES_CIPHER_OTP_XOR_ENABLE,
}ES_CIPHER_OTP_XOR_E;
/** @} */  /** <!-- ==== Structure Definition End ==== */


#define ES_CIPHER_Open(ES_VOID) ES_CIPHER_Init(ES_VOID);
#define ES_CIPHER_Close(ES_VOID) ES_CIPHER_DeInit(ES_VOID);

/******************************* API Declaration *****************************/
/** \addtogroup      CIPHER */
/** @{ */  /** <!-- [CIPHER] */
/* ---CIPHER---*/
/**
\attention \n
This API is used to start the cipher device.
\param N/A
\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_FAILED_INIT  The cipher device fails to be initialized.
\see \n
N/A
*/
ES_S32 ES_CIPHER_Init(ES_VOID);

/**
\brief  Deinit the cipher device.
\attention \n
This API is used to stop the cipher device. If this API is called repeatedly, ES_SUCCESS is returned, but only the first operation takes effect.

\param N/A
\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\see \n
N/A
*/
ES_S32 ES_CIPHER_DeInit(ES_VOID);

/**
\brief Obtain a cipher handle for symmetric encryption and decryption.

\param[out] phCipher Cipher handle
\param[in] cipher attributes
\retval ::ES_SUCCESS Call this API successful.
\retval ::ES_FAILURE Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_SERVICE_FAILED  Failed to create handle.
\see \n
N/A
*/
ES_S32 ES_CIPHER_CreateHandle(ES_HANDLE* phCipher, ES_CIPHER_CTRL_S *pstCipherAttr);

/**
\attention \n
This API is used to destroy an existing cipher handle.

\param[in] hCipher Cipher handle
\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_FAILED_RELEASEHANDLE Failed to destroy handle.
\see \n
N/A
*/
ES_S32 ES_CIPHER_DestroyHandle(ES_HANDLE hCipher);

/**
\brief Performs encryption.

\attention \n
This API is used to perform encryption by using the cipher module.
The length of the encrypted data should be a multiple of 8 in TDES mode and 16 in AES mode. Besides, the length can not be bigger than 0xFFFFF.After this operation, the result will affect next operation.If you want to remove vector, you need to config IV(config pstCtrl->stChangeFlags.bit1IV with 1) by transfering ES_CIPHER_ConfigHandle.
\param[in] hCipher Cipher handle
\param[in] pu8SrcData: buffer of the source data.
\param[out] pu8DestData: buffer of the target data
\param[in] u32ByteLength   Length of the source data
\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_FAILED_GETHANDLE  Failed to get the handle by the handle ID.
\retval ::ES_ERR_CIPHER_IOVA  Failed to get the IOVA for the input buffer.
\retval ::ES_ERR_CIPHER_HW_ERROR  Failed to send the message to the CIPHER HW.
\retval ::ES_ERR_CIPHER_SERVICE_FAILED  Encypt service rerturns error.
\see \n
N/A
*/
ES_S32 ES_CIPHER_Encrypt(ES_HANDLE hCipher, ES_U8 *pu8SrcData, ES_U8 *pu8DestData, ES_U32 u32ByteLength);

/**
\brief Performs decryption.

\attention \n
This API is used to perform decryption by using the cipher module.
The length of the decrypted data should be a multiple of 8 in TDES mode and 16 in AES mode. Besides, the length can not be bigger than 0xFFFFF.After this operation, the result will affect next operation.If you want to remove vector, you need to config IV(config pstCtrl->stChangeFlags.bit1IV with 1) by transfering ES_CIPHER_ConfigHandle.
\param[in] hCipher Cipher handle.
\param[in] pu8SrcData: buffer of the source data.
\param[out] pu8DestData: buffer of the target data
\param[in] u32ByteLength Length of the source data
\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_FAILED_GETHANDLE  Failed to get the handle by the handle ID.
\retval ::ES_ERR_CIPHER_IOVA  Failed to get the IOVA for the input buffer.
\retval ::ES_ERR_CIPHER_HW_ERROR  Failed to send the message to the CIPHER HW.
\retval ::ES_ERR_CIPHER_SERVICE_FAILED  Encypt service rerturns error.
\see \n
N/A
*/
ES_S32 ES_CIPHER_Decrypt(ES_HANDLE hCipher, ES_U8 *pu8SrcData, ES_U8 *pu8DestData, ES_U32 u32ByteLength);


/**
\brief Get the cipher control information.

\param[in] hCipher Cipher handle.
\param[in] pstCtrl Cipher control information.
\retval ::ES_SUCCESS Call this API successful.
\retval ::ES_FAILURE Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_FAILED_GETHANDLE  Failed to find handle.
\see \n
N/A
*/
ES_S32 ES_CIPHER_GetHandleConfig(ES_HANDLE hCipher, ES_CIPHER_CTRL_S* pstCtrl);

/**
\brief Select the HASH algorithm and return the HASH handle

\attention \n
N/A

\param[in] pstHashCtrl: The hash calculating structure input.
\param[out] pHashHandle: The output hash handle.
\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_SERVICE_FAILED  Failed to create handle.

\see \n
N/A
*/
ES_S32 ES_CIPHER_HashInit(ES_CIPHER_HASH_CTRL_S *pstHashCtrl, ES_HANDLE *pHashHandle);

/**
\brief Calculate the hash, the maximum size of the input data is 1GB. It's not supported for multi-package

\attention \n
N/A

\param[in] hHashHandle:  Hash handle.
\param[in] pu8InputData:  The input data buffer.
\param[in] u32InputDataLen:  The max value is 1GB
\param[out] pu8OutputHash: The output hash result
\param[out] pu32Hlen: The length of the output hash result
\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_IOVA  Failed to get the IOVA for the input buffer.
\retval ::ES_ERR_CIPHER_HW_ERROR  Failed to send the message to the CIPHER HW.
\retval ::ES_ERR_CIPHER_SERVICE_FAILED  The service return error.

\see \n
N/A
*/
ES_S32 ES_CIPHER_HashFinish(ES_HANDLE hHashHandle, ES_U8 *pu8InputData, ES_U32 u32InputDataLen, ES_U8 *pu8OutputHash, ES_U32 *pu32Hlen);

/**
\brief Get the random number.

\attention \n
This API is used to obtain the random number from the hardware.

\param[in] pu8RandomNumber Point to the random number.
\param[in] u32ByteLength:  The lenght of the random data.
\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_HW_ERROR  Failed to send the message to the CIPHER HW.
\retval ::ES_ERR_CIPHER_SERVICE_FAILED  The service return error.

\see \n
N/A
*/
ES_S32 ES_CIPHER_GetRandomNumber(ES_U8 *pu8RandomNumber, ES_U32 u32ByteLength);


/**
\brief RSA encryption a plaintext with a RSA public key.

\attention \n
N/A

\param[in] pstRsaEnc:   encryption struct.
\param[in] pu8Input:    input data to be encrypted
\param[in] u32InLen:   length of input data to be encrypted
\param[out] pu8Output: output encrypted data
\param[out] pu32OutLen: length of output encrypted data

\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_IOVA  Failed to get the IOVA for the input/output buffer.
\retval ::ES_ERR_CIPHER_HW_ERROR  Failed to send the message to the CIPHER HW.
\retval ::ES_ERR_CIPHER_SERVICE_FAILED  The service return error.

\see \n
N/A
*/
ES_S32 ES_CIPHER_RsaPublicEncrypt(ES_CIPHER_RSA_PUB_CTRL_S *pstRsaEnc,
                                  ES_U8 *pu8Input, ES_U32 u32InLen,
                                  ES_U8 *pu8Output, ES_U32 *pu32OutLen);

/**
\brief RSA decryption a ciphertext with a RSA private key.

\attention \n
N/A

\param[in] pstRsaDec:   decryption struct.
\param[in] pu8Input:    input data to be decrypted
\param[out] u32InLen:   length of input data to be decrypted
\param[out] pu8Output:  output decrypted data
\param[out] pu32OutLen: length of output decrypted data

\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_IOVA  Failed to get the IOVA for the input/output buffer.
\retval ::ES_ERR_CIPHER_HW_ERROR  Failed to send the message to the CIPHER HW.
\retval ::ES_ERR_CIPHER_SERVICE_FAILED  The service return error.

\see \n
N/A
*/
ES_S32 ES_CIPHER_RsaPrivateDecrypt(ES_CIPHER_RSA_PRI_CTRL_S *pstRsaDec,
				     ES_U8 *pu8Input, ES_U32 u32InLen,
				     ES_U8 *pu8Output, ES_U32 *pu32OutLen);

/**
\brief RSA encryption a plaintext with a RSA private key.

\attention \n
N/A

\param[in] pstRsaSign:   encryption struct.
\param[in] pu8Input:     input data to be encryption
\param[out] u32InLen:   length of input data to be encryption
\param[out] pu8Output:  output data to be encryption
\param[out] pu32OutLen: length of output data to be encryption

\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_IOVA  Failed to get the IOVA for the input/output buffer.
\retval ::ES_ERR_CIPHER_HW_ERROR  Failed to send the message to the CIPHER HW.
\retval ::ES_ERR_CIPHER_SERVICE_FAILED  The service return error.

\see \n
N/A
*/
ES_S32 ES_CIPHER_RsaPrivateEncrypt(ES_CIPHER_RSA_PRI_CTRL_S *pstRsaEnc,
				     ES_U8 *pu8Input, ES_U32 u32InLen,
				     ES_U8 *pu8Output, ES_U32 *pu32OutLen);

/**
\brief RSA decryption a ciphertext with a RSA public key.

\attention \n
N/A

\param[in] pstRsaDec:   decryption struct.
\param[in] pu8Input:   input data to be decryption
\param[out] u32InLen:   length of input data to be decryption
\param[out] pu8Output: output data to be decryption
\param[out] pu32OutLen: length of output data to be decryption

\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_IOVA  Failed to get the IOVA for the input/output buffer.
\retval ::ES_ERR_CIPHER_HW_ERROR  Failed to send the message to the CIPHER HW.
\retval ::ES_ERR_CIPHER_SERVICE_FAILED  The service return error.

\see \n
N/A
*/
ES_S32 ES_CIPHER_RsaPublicDecrypt(ES_CIPHER_RSA_PUB_CTRL_S *pstRsaDec,
				  ES_U8 *pu8Input, ES_U32 u32InLen,
				  ES_U8 *pu8Output, ES_U32 *pu32OutLen);


/**
\brief Perform signature verification on the NSIGN bin file, output the signature struct info if success.

\attention \n
N/A

\param[in] pstNsignVerify:	NSIGN verification struct.
\param[in] pu8InData:		input payload to be signature verification
\param[in] u32InDataLen:	length of input data to be signature
\param[in] pu8InSign:		encrypted signature struct info
\param[in] pu32InSignLen:	length of encrypted signature struct info
\param[out] pu8Output:		decrypted signature struct info
\param[out] pu32OutLen:		length of the decrypted signature struct info

\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_IOVA  Failed to get the IOVA for the input/output buffer.
\retval ::ES_ERR_CIPHER_HW_ERROR  Failed to send the message to the CIPHER HW.
\retval ::ES_ERR_CIPHER_SERVICE_FAILED  The service return error.

\see \n
N/A
*/
ES_S32 ES_CIPHER_NsignVerify(ES_CIPHER_NSIGN_VERIFY_S *pstNsignVerify,
				ES_U8 *pu8InData, ES_U32 u32InDataLen,
				ES_U8 *pu8InSign, ES_U32 u32InSignLen,
				ES_U8 *pu8Output, ES_U32 *pu32OutLen);



/**
\brief Perform the signature verification and decryption the image on the NSIGN bin file.

\attention \n
N/A

\param[in] pstNsignVerify:	NSIGN verification struct.
\param[in] pu8Input:		input context to be signature verification
\param[in] u32InLen:		length of input context to be signature
\param[in] pu8InSign:		message of signature
\param[in] pu32InSignLen:	length of message of signature
\param[out] pu8Output:		decrypted image/data
\param[out] pu32OutLen:		length of the decrypted image/data

\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_IOVA  Failed to get the IOVA for the input/output buffer.
\retval ::ES_ERR_CIPHER_HW_ERROR  Failed to send the message to the CIPHER HW.
\retval ::ES_ERR_CIPHER_SERVICE_FAILED  The service return error.

\see \n
N/A
*/
ES_S32 ES_CIPHER_ImageDec(ES_CIPHER_NSIGN_VERIFY_S *pstNsignVerify,
				ES_U8 *pu8InData, ES_U32 u32InDataLen,
				ES_U8 *pu8InSign, ES_U32 u32InSignLen,
				ES_U8 *pu8Output, ES_U32 *pu32OutLen);


/**
\brief ECDH public and private Key Generation.

\attention \n
N/A

\param[in] enEccCurve:		Eliptic curve type.
\param[out] pstEcdhKey:		ES_CIPHER_ECDH_KEY_S struct

\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_HW_ERROR  Failed to send the message to the CIPHER HW.
\retval ::ES_ERR_CIPHER_SERVICE_FAILED  The service return error.

\see \n
N/A
*/
ES_S32 ES_CIPHER_EcdhKeyGen(ES_CIHPER_ECC_CURVE_TYPE_E enEccCurve, ES_CIPHER_ECDH_KEY_S *pstEcdhKey);


/**
\brief ECDH shared public key agreement.

\attention \n
N/A

\param[in] enEccCurve:			Eliptic curve type.
\param[in] pstRemoteEcdhPubKey:		remote ECC public key struct.
\param[in] pstLocalEcdhPriKey:		local ECC private key struct.
\param[out] pstOutSharedEcdhPubKey:	agreement shared ECC public key struct

\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_HW_ERROR  Failed to send the message to the CIPHER HW.
\retval ::ES_ERR_CIPHER_SERVICE_FAILED  The service return error.

\see \n
N/A
*/
ES_S32 ES_CIPHER_EcdhKeyAgree(ES_CIHPER_ECC_CURVE_TYPE_E enEccCurve,
				ES_CIPHER_ECDH_PUB_KEY_S *pstRemoteEcdhPubKey,
				ES_CIPHER_ECDH_PRI_KEY_S *pstLocalEcdhPriKey,
				ES_CIPHER_ECDH_PUB_KEY_S *pstOutSharedEcdhPubKey);


/**
\brief ECDH key derivation.Output a key that can be used as a symmetric algorithm key

\attention \n
N/A

\param[in] enEccCurve:			Eliptic curve type.
\param[in] pstSharedEcdhPubKey:		agreement shared ECC public key struct.
\param[in] u32Rand	:		random data.
\param[out] pu8OutSymcKey:		symc key, fixed len 64Bytes

\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_HW_ERROR  Failed to send the message to the CIPHER HW.
\retval ::ES_ERR_CIPHER_SERVICE_FAILED  The service return error.

\see \n
N/A
*/
ES_S32 ES_CIPHER_EcdhKeyDerivation(ES_CIHPER_ECC_CURVE_TYPE_E enEccCurve,
					ES_CIPHER_ECDH_PUB_KEY_S *pstSharedEcdhPubKey,
					ES_U32 u32Rand, ES_U8 *pu8OutSymcKey);


/**
\brief firmware download & execute.

\attention \n
N/A

\param[in] pstNsignVerify:	NSIGN verification struct.
\param[in] pu8Input:		firmware image
\param[in] u32InLen:		length of firmware
\param[in] pu8InSign:		message of signature
\param[in] pu32InSignLen:	length of message of signature
\param[out] ps32FirmRetVal:	return value of firmware execution


\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.

\see \n
N/A
*/
ES_S32 ES_CIPHER_FirmwareDownload(ES_CIPHER_NSIGN_VERIFY_S *pstNsignVerify,
				ES_U8 *pu8InData, ES_U32 u32InDataLen,
				ES_U8 *pu8InSign, ES_U32 u32InSignLen,  ES_S32 *ps32FirmRetVal);


/**
\brief public key download, i.e update the public key in the RAM.

\attention \n
N/A

\param[in] pstNsignVerify:	NSIGN verification struct.
\param[in] pu8InData:		ES_CIPHER_RSA_PUB_KEY_S or ES_CIPHER_ECDH_PUB_KEY_S struct, it might be cleartext or ciphertxt
\param[in] u32InDataLen:		length of pubkey struct
\param[in] pu8InSign:		message of signature
\param[in] pu32InSignLen:	length of message of signature


\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_IOVA  Failed to get the IOVA for the input buffer.
\retval ::ES_ERR_CIPHER_HW_ERROR  Failed to send the message to the CIPHER HW.
\retval ::ES_ERR_CIPHER_SERVICE_FAILED  The service return error.

\see \n
N/A
*/
ES_S32 ES_CIPHER_PubKeyDownload(ES_CIPHER_NSIGN_VERIFY_S *pstNsignVerify,
				ES_U8 *pu8InData, ES_U32 u32InDataLen,
				ES_U8 *pu8InSign, ES_U32 u32InSignLen);


/**
\brief loadable service code register.

\attention \n
N/A

\param[in] pstNsignVerify:	NSIGN verification struct.
\param[in] pu8InData:		service code
\param[in] u32InDataLen:	length of the service code
\param[in] pu8InSign:		message of signature
\param[in] pu32InSignLen:	length of message of signature
\param[out] pu32ServiceId:	service ID

\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_IOVA  Failed to get the IOVA for the input buffer.
\retval ::ES_ERR_CIPHER_HW_ERROR  Failed to send the message to the CIPHER HW.
\retval ::ES_ERR_CIPHER_SERVICE_FAILED  The service return error.

\see \n
N/A
*/
ES_S32 ES_CIPHER_LoadableRegister(ES_CIPHER_NSIGN_VERIFY_S *pstNsignVerify,
					ES_U8 *pu8InData, ES_U32 u32InDataLen,
					ES_U8 *pu8InSign, ES_U32 u32InSignLen, ES_U32 *pu32ServiceId);



/**
\brief loadable service code unregister.

\attention \n
N/A

\param[in] pstNsignVerify:	NSIGN verification struct.
\param[in] u32ServiceId:	service ID that was registerd previously

\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_HW_ERROR  Failed to send the message to the CIPHER HW.
\retval ::ES_ERR_CIPHER_SERVICE_FAILED  The service return error.

\see \n
N/A
*/
ES_S32 ES_CIPHER_LoadableUnRegister(ES_CIPHER_NSIGN_VERIFY_S *pstNsignVerify, ES_U32 u32ServiceId);


/**
\brief loadable service ioctl call.

\attention \n
N/A

\param[in] pstNsignVerify:	NSIGN verification struct.
\param[in] u32ServiceId:	service ID that was registerd previously
\param[in] u32Cmd:		ioctl cmd
\param[in] u32ParamSize:	sizeof the parameter
\param[in] pu8Param:		parameter


\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_HW_ERROR  Failed to send the message to the CIPHER HW.
\retval ::ES_ERR_CIPHER_SERVICE_FAILED  The service return error.

\see \n
N/A
*/
ES_S32 ES_CIPHER_LoadableIoctl(ES_CIPHER_NSIGN_VERIFY_S *pstNsignVerify, ES_U32 u32ServiceId,
				ES_U32 u32Cmd, ES_U32 u32ParamSize, ES_U8 *pu8Param);


/**
\brief OTP Read function.

\attention \n
N/A

\param[in] enXorEnable:		XOR Enable or disable.
\param[in] u32OtpAddr:		OTP addr to read
\param[in] u32OtpDataSize:	OTP size to read
\param[out] pu8OutData:		data that was read
\param[out] pu32OutLen:		actual length of the data that was read

\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_HW_ERROR  Failed to send the message to the CIPHER HW.
\retval ::ES_ERR_CIPHER_SERVICE_FAILED  The service return error.

\see \n
N/A
*/
ES_S32 ES_CIPHER_OtpRead(ES_CIPHER_OTP_XOR_E enXorEnable, ES_U32 u32OtpAddr,
				ES_U32 u32OtpDataSize, ES_U8 *pu8OutData, ES_U32 *pu32OutLen);


/**
\brief OTP Write function.
\attention \n
N/A

\param[in] enXorEnable:		XOR Enable or disable.
\param[in] u32OtpAddr:		OTP addr to write
\param[in] u32OtpDataSize:	OTP size to write
\param[in] pu8OtpData:		OTP data to write
\param[in] pu8Mask:		OTP mask
\param[out] pu8OutData:		OTP data that was written finally
\param[out] pu32OutLen:		actual length of the data that was wrote

\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_IOVA  Failed to get the IOVA for the input buffer.
\retval ::ES_ERR_CIPHER_HW_ERROR  Failed to send the message to the CIPHER HW.
\retval ::ES_ERR_CIPHER_SERVICE_FAILED  The service return error.


\see \n
N/A
*/
ES_S32 ES_CIPHER_OtpWrite(ES_CIPHER_OTP_XOR_E enXorEnable,
				ES_U32 u32OtpAddr, ES_U32 u32OtpDataSize,
				ES_U8 *pu8OtpData, ES_U8 *pu8Mask, ES_U8 *pu8OutData, ES_U32 *pu32OutLen);

/**
\brief parse the bin file generated by NSIGN Tool, i.e get the decrypted sign and decrypted image from the file.

\attention \n
N/A

\param[in] pu8InFileData	the data address of the file generated by NSIGN tool
\param[in] u32InFileDataLen	data length of file
\param[out] pstOutNsignVerify	NSIGN verification struct
\param[out] ppu8OutSignAddr:	the sign address in the file
\param[out] pu32OutSignLen:	length of the signature
\param[out] ppu8OutPayloadAddr:	payload address
\param[out] pu32OutPayloadLen:	length of the payload


\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.

\see \n
N/A
*/
ES_S32 ES_CIPHER_NsignFileParse(ES_U8 *pu8InFileData, ES_U32 u32InFileDataLen,
					ES_CIPHER_NSIGN_VERIFY_S *pstOutNsignVerify,
					ES_U8 **ppu8OutSignAddr, ES_U32 *pu32OutSignLen,
					ES_U8 **ppu8OutPayloadAddr, ES_U32 *pu32OutPayloadLen);

/**
\brief parse the bin file generated by NSIGN Tool, i.e get the decrypted sign and decrypted image from the file.

\attention \n
N/A

\param[in] pstRsaSign:		RSA private encryption ctrl info
\param[in] pu8InData:		input context to be signature.
\param[in] u32InLen:		length of input context to be signature
\param[out] pu8OutSign:		output message of signature
\param[out] pu32OutSignLen:	length of message of signature
\retval ::ES_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.
\retval ::ES_ERR_CIPHER_INVALID_PARA  The parameter is invalid.
\retval ::ES_ERR_CIPHER_IOVA  Failed to get the IOVA for the input/output buffer.
\retval ::ES_ERR_CIPHER_HW_ERROR  Failed to send the message to the CIPHER HW.
\retval ::ES_ERR_CIPHER_SERVICE_FAILED  The service return error.

\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.

\see \n
N/A
*/
ES_S32 ES_CIPHER_RsaSign(ES_CIPHER_RSA_PRI_CTRL_S *pstRsaSign,
				ES_U8 *pu8InData, ES_U32  u32InDataLen,
				ES_U8 *pu8OutSign, ES_U32 *pu32OutSignLen);

/**
\brief RSA signature verification a context with appendix, where a signer's RSA public key is used.

\attention \n
N/A

\param[in] pstRsaVerify:    RSA public decryption ctrl info
\param[in] pu8Input:       input context to be signature verification, maybe null
\param[in] u32InLen:        length of input context to be signature
\param[in] pu8InSign:      message of signature
\param[in] pu32InSignLen:   length of message of signature

\retval ::ES_SUCCESS  Call this API successful.
\retval ::ES_FAILURE  Call this API fails.

\see \n
N/A
*/
ES_S32 ES_CIPHER_RsaVerify(ES_CIPHER_RSA_PUB_CTRL_S *pstRsaVerify,
				ES_U8 *pu8InData, ES_U32 u32InDataLen,
				ES_U8 *pu8InSign, ES_U32 u32InSignLen);
/** @} */  /** <!-- ==== API declaration end ==== */

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif /* __cplusplus */

#endif /* __ES_CIPHER_H__ */

